@@ -7,15 +7,16 @@ describe: 缠论分型、笔的识别
 """
 import os
 import webbrowser
-import traceback
 import numpy as np
+from loguru import logger
 from typing import List, Callable
 from collections import OrderedDict
+from czsc.enum import Mark, Direction
+from czsc.objects import BI, FX, RawBar, NewBar
+from czsc.utils.echarts_plot import kline_pro
+from czsc import envs
 
-from .enum import Mark, Direction
-from .objects import BI, FX, RawBar, NewBar, Signal
-from .utils.echarts_plot import kline_pro
-from . import envs
+logger.disable('czsc.analyze')
 
 
 def remove_include(k1: NewBar, k2: NewBar, k3: RawBar):
@@ -84,8 +85,17 @@ def check_fxs(bars: List[NewBar]) -> List[FX]:
             # 这里可能隐含Bug，默认情况下，fxs本身是顶底交替的，但是对于一些特殊情况下不是这样，这是不对的。
             # 临时处理方案，强制要求fxs序列顶底交替
             if len(fxs) >= 2 and fx.mark == fxs[-1].mark:
-                fxs.pop()
-            fxs.append(fx)
+                if envs.get_verbose():
+                    logger.info(f"\n\ncheck_fxs: 输入数据错误{'+' * 100}")
+                    logger.info(f"当前：{fx.mark}, 上个：{fxs[-1].mark}")
+                    for bar in fx.raw_bars:
+                        logger.info(f"{bar}\n")
+
+                    logger.info(f'last fx raw bars: \n')
+                    for bar in fxs[-1].raw_bars:
+                        logger.info(f"{bar}\n")
+            else:
+                fxs.append(fx)
     return fxs
 
 
@@ -110,7 +120,7 @@ def check_bi(bars: List[NewBar], benchmark: float = None):
                 return None, bars
 
             fx_b = fxs_b[0]
-            for fx in fxs_b:
+            for fx in fxs_b[1:]:
                 if fx.high >= fx_b.high:
                     fx_b = fx
 
@@ -127,7 +137,7 @@ def check_bi(bars: List[NewBar], benchmark: float = None):
         else:
             raise ValueError
     except:
-        traceback.print_exc()
+        logger.exception("笔识别错误")
         return None, bars
 
     bars_a = [x for x in bars if fx_a.elements[0].dt <= x.dt <= fx_b.elements[2].dt]
@@ -159,65 +169,20 @@ def check_bi(bars: List[NewBar], benchmark: float = None):
         return None, bars
 
 
-def signals_counter(signals_list) -> OrderedDict:
-    """信号连续出现次数记录
-
-    :param signals_list: 存储的信号列表
-        数据样例：
-            signals_list = [
-                {"dt": '2020-01-01', "日线_倒1K_SMA5多空": "空头_任意_任意_0"},
-                {"dt": '2020-01-02', "日线_倒1K_SMA5多空": "多头_任意_任意_0"},
-                {"dt": '2020-01-03', "日线_倒1K_SMA5多空": "多头_任意_任意_0"},
-                {"dt": '2020-01-04', "日线_倒1K_SMA5多空": "多头_任意_任意_0"},
-                {"dt": '2020-01-05', "日线_倒1K_SMA5多空": "多头_任意_任意_0"},
-                {"dt": '2020-01-06', "日线_倒1K_SMA5多空": "多头_任意_任意_0"},
-            ]
-    :return: 信号连续次数
-    """
-    if not signals_list:
-        return OrderedDict()
-
-    signals = [Signal(f"{k}_{v}") for k, v in signals_list[-1].items()
-               if len(k.split("_")) == 3 and "连续次数" not in k]
-
-    s = OrderedDict()
-    for signal in signals:
-        k1 = signal.k1
-        k2 = f"{signal.k2}#{signal.k3}"
-        k3 = "连续次数"
-        seq = [signal.is_match(x) for x in signals_list]
-        assert seq[-1], "最后一个信号匹配结果必须为 True"
-
-        n = 0
-        for x in seq:
-            if x:
-                n += 1
-            else:
-                n = 0
-        assert n >= 1, "连续次数小于1，不合逻辑"
-
-        signal_c = Signal(k1=k1, k2=k2, k3=k3, v1=f"{n}次")
-        s[signal_c.key] = signal_c.value
-
-    return s
-
-
 class CZSC:
     def __init__(self,
                  bars: List[RawBar],
                  get_signals: Callable = None,
                  max_bi_num=envs.get_max_bi_num(),
-                 signals_n: int = 0):
+                 ):
         """
 
         :param bars: K线数据
         :param max_bi_num: 最大允许保留的笔数量
         :param get_signals: 自定义的信号计算函数
-        :param signals_n: 缓存n个历史时刻的信号，0 表示不缓存；缓存的数据，主要用于计算信号连续次数
         """
         self.verbose = envs.get_verbose()
         self.max_bi_num = max_bi_num
-        self.signals_n = signals_n
         self.bars_raw: List[RawBar] = []  # 原始K线序列
         self.bars_ubi: List[NewBar] = []  # 未完成笔的无包含K线序列
         self.bi_list: List[BI] = []
@@ -225,7 +190,6 @@ class CZSC:
         self.freq = bars[0].freq
         self.get_signals = get_signals
         self.signals = None
-        self.signals_list = []
 
         for bar in bars:
             self.update(bar)
@@ -240,7 +204,7 @@ class CZSC:
 
         # 查找笔
         if not self.bi_list:
-            # 第一个笔的查找
+            # 第一笔的查找
             fxs = check_fxs(bars_ubi)
             if not fxs:
                 return
@@ -259,29 +223,27 @@ class CZSC:
             self.bars_ubi = bars_ubi_
             return
 
-        last_bi = self.bi_list[-1]
-
-        # 如果上一笔被破坏，将上一笔的bars与bars_ubi进行合并
-        if (last_bi.direction == Direction.Up and bars_ubi[-1].high > last_bi.high) \
-                or (last_bi.direction == Direction.Down and bars_ubi[-1].low < last_bi.low):
-            bars_ubi_a = last_bi.bars[:-1] + [x for x in bars_ubi if x.dt >= last_bi.bars[-1].dt]
-            self.bi_list.pop(-1)
-        else:
-            bars_ubi_a = bars_ubi
-
-        if self.verbose and len(bars_ubi_a) > 100:
-            print(f"czsc_update_bi: {self.symbol} - {self.freq} - {bars_ubi_a[-1].dt} 未完成笔延伸数量: {len(bars_ubi_a)}")
+        if self.verbose and len(bars_ubi) > 100:
+            logger.info(f"{self.symbol} - {self.freq} - {bars_ubi[-1].dt} 未完成笔延伸数量: {len(bars_ubi)}")
 
         if envs.get_bi_change_th() > 0.5 and len(self.bi_list) >= 5:
-            benchmark = min(last_bi.power_price, np.mean([x.power_price for x in self.bi_list[-5:]]))
+            benchmark = min(self.bi_list[-1].power_price, np.mean([x.power_price for x in self.bi_list[-5:]]))
         else:
             benchmark = None
 
-        bi, bars_ubi_ = check_bi(bars_ubi_a, benchmark)
+        bi, bars_ubi_ = check_bi(bars_ubi, benchmark)
         self.bars_ubi = bars_ubi_
         if isinstance(bi, BI):
             self.bi_list.append(bi)
 
+        # 后处理：如果当前笔被破坏，将当前笔的bars与bars_ubi进行合并，并丢弃
+        last_bi = self.bi_list[-1]
+        bars_ubi = self.bars_ubi
+        if (last_bi.direction == Direction.Up and bars_ubi[-1].high > last_bi.high) \
+                or (last_bi.direction == Direction.Down and bars_ubi[-1].low < last_bi.low):
+            self.bars_ubi = last_bi.bars[:-1] + [x for x in bars_ubi if x.dt >= last_bi.bars[-1].dt]
+            self.bi_list.pop(-1)
+
     def update(self, bar: RawBar):
         """更新分析结果
 
@@ -292,10 +254,16 @@ class CZSC:
             self.bars_raw.append(bar)
             last_bars = [bar]
         else:
+            # 当前 bar 是上一根 bar 的时间延伸
             self.bars_raw[-1] = bar
-            last_bars = self.bars_ubi[-1].elements
-            last_bars[-1] = bar
-            self.bars_ubi.pop(-1)
+            if len(self.bars_ubi) >= 3:
+                edt = self.bars_ubi[-2].dt
+                self.bars_ubi = [x for x in self.bars_ubi if x.dt <= edt]
+                last_bars = [x for x in self.bars_raw[-50:] if x.dt > edt]
+            else:
+                last_bars = self.bars_ubi[-1].elements
+                last_bars[-1] = bar
+                self.bars_ubi.pop(-1)
 
         # 去除包含关系
         bars_ubi = self.bars_ubi
@@ -315,6 +283,8 @@ class CZSC:
 
         # 更新笔
         self.__update_bi()
+
+        # 根据最大笔数量限制完成 bi_list, bars_raw 序列的数量控制
         self.bi_list = self.bi_list[-self.max_bi_num:]
         if self.bi_list:
             sdt = self.bi_list[0].fx_a.elements[0].dt
@@ -325,12 +295,9 @@ class CZSC:
                     break
             self.bars_raw = self.bars_raw[s_index:]
 
+        # 如果有信号计算函数，则进行信号计算
         if self.get_signals:
             self.signals = self.get_signals(c=self)
-            if self.signals_n > 0:
-                self.signals_list.append(self.signals)
-                self.signals_list = self.signals_list[-self.signals_n:]
-                self.signals.update(signals_counter(self.signals_list))
         else:
             self.signals = OrderedDict()
 
@@ -346,9 +313,7 @@ class CZSC:
         if len(self.bi_list) > 0:
             bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in self.bi_list] + \
                  [{'dt': self.bi_list[-1].fx_b.dt, "bi": self.bi_list[-1].fx_b.fx}]
-            fx = []
-            for bi_ in self.bi_list:
-                fx.extend([{'dt': x.dt, "fx": x.fx} for x in bi_.fxs[1:]])
+            fx = [{'dt': x.dt, "fx": x.fx} for x in self.fx_list]
         else:
             bi = None
             fx = None
@@ -392,4 +357,22 @@ class CZSC:
                 return self.bi_list[:-1]
         return self.bi_list
 
+    @property
+    def ubi_fxs(self) -> List[FX]:
+        """返回当下基本确认完成的笔列表"""
+        if not self.bars_ubi:
+            return []
+        else:
+            return check_fxs(self.bars_ubi)
 
+    @property
+    def fx_list(self) -> List[FX]:
+        """返回当下基本确认完成的笔列表"""
+        fxs = []
+        for bi_ in self.bi_list:
+            fxs.extend(bi_.fxs[1:])
+        ubi = self.ubi_fxs
+        for x in ubi:
+            if not fxs or x.dt > fxs[-1].dt:
+                fxs.append(x)
+        return fxs
