@@ -5,22 +5,25 @@ email: zeng_bin8888@163.com
 create_dt: 2021/11/14 12:39
 describe: 从任意周期K线开始合成更高周期K线的工具类
 """
+import pandas as pd
 from datetime import datetime, timedelta
-from typing import List
-from ..objects import RawBar, Freq
+from typing import List, Union, AnyStr
+from czsc.objects import RawBar, Freq
 
 
-def freq_end_time(dt: datetime, freq: Freq) -> datetime:
+def freq_end_time(dt: datetime, freq: Union[Freq, AnyStr]) -> datetime:
     """获取 dt 对应的K线周期结束时间
 
     :param dt: datetime
     :param freq: Freq
     :return: datetime
     """
+    if not isinstance(freq, Freq):
+        freq = Freq(freq)
     dt = dt.replace(second=0, microsecond=0)
 
     if freq in [Freq.F1, Freq.F5, Freq.F15, Freq.F30, Freq.F60]:
-        m = int(freq.value.strip("分钟"))
+        m = int(str(freq.value).strip("分钟"))
         if m < 60:
             if (dt.hour == 15 and dt.minute == 0) or (dt.hour == 11 and dt.minute == 30):
                 return dt
@@ -76,6 +79,54 @@ def freq_end_time(dt: datetime, freq: Freq) -> datetime:
     return dt
 
 
+def resample_bars(df: pd.DataFrame, target_freq: Union[Freq, AnyStr], raw_bars=True, **kwargs):
+    """将df中的K线序列转换为目标周期的K线序列
+
+    :param df: 原始K线数据，必须包含以下列：symbol, dt, open, close, high, low, vol, amount。样例如下：
+               symbol                  dt     open    close     high      low  \
+        0  000001.XSHG 2015-01-05 09:31:00  3258.63  3259.69  3262.85  3258.63
+        1  000001.XSHG 2015-01-05 09:32:00  3258.33  3256.19  3259.55  3256.19
+        2  000001.XSHG 2015-01-05 09:33:00  3256.10  3257.50  3258.42  3256.10
+        3  000001.XSHG 2015-01-05 09:34:00  3259.33  3261.76  3261.76  3257.98
+        4  000001.XSHG 2015-01-05 09:35:00  3261.71  3264.88  3265.48  3261.71
+                  vol        amount
+        0  1333523100  4.346872e+12
+        1   511386100  1.665170e+12
+        2   455375200  1.483385e+12
+        3   363393800  1.185303e+12
+        4   402854600  1.315272e+12
+    :param target_freq: 目标周期
+    :param raw_bars: 是否将转换后的K线序列转换为RawBar对象
+    :return: 转换后的K线序列
+    """
+    if not isinstance(target_freq, Freq):
+        target_freq = Freq(target_freq)
+
+    k_cols = ['symbol', 'dt', 'open', 'close', 'high', 'low', 'vol', 'amount']
+    df = df[k_cols]
+    df['freq_edt'] = df['dt'].apply(lambda x: freq_end_time(x, target_freq))
+    dfk1 = df.groupby('freq_edt').agg(
+        {'symbol': 'first', 'dt': 'last', 'open': 'first', 'close': 'last', 'high': 'max',
+         'low': 'min', 'vol': 'sum', 'amount': 'sum', 'freq_edt': 'last'})
+    dfk1.reset_index(drop=True, inplace=True)
+    dfk1['dt'] = dfk1['freq_edt']
+    dfk1 = dfk1[k_cols]
+
+    if raw_bars:
+        _bars = []
+        for i, row in enumerate(dfk1.to_dict("records"), 1):
+            row.update({'id': i, 'freq': target_freq})
+            _bars.append(RawBar(**row))
+
+        if df['dt'].iloc[-1] < _bars[-1].dt:
+            # 清除最后一根未完成的K线
+            _bars.pop()
+
+        return _bars
+    else:
+        return dfk1
+
+
 class BarGenerator:
     """使用日线合成周线、月线、季线"""
 
@@ -147,14 +198,9 @@ class BarGenerator:
             self.bars[freq.value].append(bar_)
 
         else:
-            if last.amount:
-                amount = last.amount + bar.amount
-            else:
-                amount = None
-
             bar_ = RawBar(symbol=bar.symbol, freq=freq, dt=freq_edt, id=last.id,
                           open=last.open, close=bar.close, high=max(last.high, bar.high),
-                          low=min(last.low, bar.low), vol=last.vol + bar.vol, amount=amount)
+                          low=min(last.low, bar.low), vol=last.vol + bar.vol, amount=last.amount + bar.amount)
             self.bars[freq.value][-1] = bar_
 
     def update(self, bar: RawBar) -> None:
