@@ -4,7 +4,9 @@ from tqdm import tqdm
 import pandas as pd
 from czsc.analyze import *
 from czsc.enum import Freq
-from czsc.signals.signals import get_default_signals, get_s_three_bi, get_s_d0_bi
+from collections import OrderedDict
+from czsc.signals.bxt import get_s_d0_bi, get_s_three_bi
+
 
 cur_path = os.path.split(os.path.realpath(__file__))[0]
 
@@ -15,6 +17,7 @@ def read_1min():
         data = pd.read_csv(f, encoding='utf-8')
 
     data['dt'] = pd.to_datetime(data['dt'])
+    data['amount'] = data['close'] * data['vol']
     records = data.to_dict('records')
 
     bars = []
@@ -28,9 +31,11 @@ def read_1min():
 def read_daily():
     file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
     kline = pd.read_csv(file_kline, encoding="utf-8")
+    kline['amount'] = kline['close'] * kline['vol']
+
     kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
     bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
-                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
+                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'], amount=row['amount'])
             for i, row in kline.iterrows()]
     return bars
 
@@ -85,20 +90,9 @@ def test_czsc_update():
     c = CZSC(bars)
     assert not c.signals
 
-    # 计算信号
-    c = CZSC(bars, get_signals=get_default_signals)
-    assert len(c.bi_list) == 50 and not c.last_bi_extend
-    assert isinstance(c.signals, OrderedDict) and len(c.signals) == 38
-
-    last_bi = c.bi_list[-1]
-    assert len(last_bi.raw_bars) == 32 and last_bi.power_price == last_bi.power
-    assert len(last_bi.fake_bis) == 11
-    assert last_bi.fake_bis[0].direction == last_bi.fake_bis[-1].direction == last_bi.direction
     # 测试自定义信号
-    c = CZSC(bars, get_signals=get_user_signals, signals_n=20)
-    assert len(c.signals) == 11
-    assert len(c.signals_list) == 20
-    assert c.signals_list[-1] == c.signals
+    c = CZSC(bars, get_signals=get_user_signals)
+    assert len(c.signals) == 7
 
     kline = [x.__dict__ for x in c.bars_raw]
     bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in c.bi_list] + \
